@@ -3,9 +3,9 @@ import enum
 import typing
 
 import dataclasses_json
+from pg_sql import SqlId, SqlObject
 
 from ..json import DataJsonFormat, ValidatingDataJsonFormat, package_json_format
-from ..sql import SqlId, SqlObject
 
 
 class JoinConsistency(enum.Enum):
@@ -13,12 +13,13 @@ class JoinConsistency(enum.Enum):
     IMMEDIATE = "immediate"
 
 
-class JoinDepMode(enum.Enum):
+class JoinJoinMode(enum.Enum):
     ASYNC = "async"
     SYNC = "sync"
 
 
-class JoinSync(enum.Enum):
+class JoinRefresh(enum.Enum):
+    INSERT = "insert"
     FULL = "full"
     UPSERT = "upsert"
 
@@ -28,11 +29,12 @@ class JoinSync(enum.Enum):
     undefined=dataclasses_json.Undefined.EXCLUDE,
 )
 @dataclasses.dataclass
-class JoinTarget:
-    key: typing.List[str]
+class JoinTargetTable:
     name: str
-    schema: typing.Optional[str] = None
     columns: typing.Optional[typing.List[str]] = None
+    key: typing.Optional[typing.List[str]] = None
+    schema: typing.Optional[str] = None
+    refresh: JoinRefresh = JoinRefresh.FULL
 
     @property
     def sql(self) -> SqlObject:
@@ -50,9 +52,10 @@ class JoinTarget:
 @dataclasses.dataclass
 class JoinTable:
     name: str
-    dep: typing.Optional[str] = None
-    dep_join: typing.Optional[str] = None
-    dep_mode: JoinDepMode = JoinDepMode.SYNC
+    join: typing.Optional[str] = None
+    join_on: typing.Optional[str] = None
+    join_mode: JoinJoinMode = JoinJoinMode.SYNC
+    join_other: typing.Optional[str] = None
     key: typing.Optional[typing.List[str]] = None
     lock_id: typing.Optional[int] = None
     schema: typing.Optional[str] = None
@@ -91,6 +94,20 @@ class JoinHook:
         )
 
 
+@dataclasses_json.dataclass_json(
+    letter_case=dataclasses_json.LetterCase.CAMEL,
+    undefined=dataclasses_json.Undefined.EXCLUDE,
+)
+@dataclasses.dataclass
+class JoinKeyColumn:
+    name: str
+    type: str
+
+    @property
+    def sql(self) -> SqlId:
+        return SqlId(self.name)
+
+
 @dataclasses_json.dataclass_json(
     letter_case=dataclasses_json.LetterCase.CAMEL,
     undefined=dataclasses_json.Undefined.EXCLUDE,
@@ -99,12 +116,13 @@ class JoinHook:
 class JoinConfig:
     id: str
     tables: typing.Dict[str, JoinTable]
-    target: JoinTarget
+    target_table: JoinTargetTable
+    key: typing.Optional[typing.List[JoinKeyColumn]] = None
+    lock: bool = False
     consistency: JoinConsistency = JoinConsistency.IMMEDIATE
-    query: typing.Optional[str] = None
+    target_query: typing.Optional[str] = "TABLE $1"
     schema: typing.Optional[str] = None
     setup: typing.Optional[JoinHook] = None
-    sync: JoinSync = JoinSync.FULL
 
 
 class JoinInvalid(Exception):
@@ -113,12 +131,9 @@ class JoinInvalid(Exception):
 
 
 def validate_join(join: JoinConfig):
-    if join.consistency == JoinConsistency.DEFERRED and join.query is None:
+    if join.consistency == JoinConsistency.DEFERRED and join.target_query is None:
         raise JoinInvalid("Deferrable mode is only used with query")
 
-    if join.target.columns is None and join.query is not None:
-        raise JoinInvalid("Query requires target column list")
-
 
 JOIN_JSON_FORMAT = package_json_format("denorm.formats", "join.json")
 
